package board;

import java.applet.Applet;
import java.awt.event.*;
import java.awt.*;
import java.util.*;
import world.*;


class Main extends Panel implements ButtonsInterface, ActionListener,
		MouseListener, MouseMotionListener, KeyListener {

	// 
	// button action
	//
	public void initializeButtons(Buttons buttons) {
		buttons.addButton("0");
		buttons.addButton("1");
		buttons.addButton("2");
		buttons.addButton("3");
	}

	public void buttonPressed(String label) {
		if (label.equals("0")) {
			World.arm.set_target_angle(0, World.arm.get_current_angle(0)+10);
		}
		if (label.equals("1")) {
			World.arm.set_target_angle(1, World.arm.get_current_angle(1)+10);
		}
		if (label.equals("2")) {
			World.arm.set_target_angle(2, World.arm.get_current_angle(2)+10);
		}
		if (label.equals("3")) {
			World.arm.set_target_angle(3, World.arm.get_current_angle(3)+10);
		}
	}

	
	//
	// mouse action
	//
	Vector2D offset;
	Vector2D feedback;
	void paint_feedback(Graphics g) {
		if (feedback != null) {
			g.setColor(Color.red);
			g.drawOval((int) feedback.x - 10, (int) feedback.y - 10, 20, 20);
		}
	}
	public void mousePressed(MouseEvent e) {
		Vector2D p = new Vector2D(e.getPoint());
		
		feedback = null;
		for (int i = 0; i < World.arm.N; i++) {
			if (Vector2D.distance(World.arm.get_joint_position(i), p) < World.arm.R) {
				offset = new Vector2D(p, World.arm.get_joint_position(i));
				feedback = Vector2D.add(p, offset);
				return;
			}
		}
	}
	public void mouseDragged(MouseEvent e) {
		Vector2D p = new Vector2D(e.getPoint());
		if (feedback != null)
			feedback.set(Vector2D.add(p, offset));
	}
	public void mouseReleased(MouseEvent e) {
		feedback = null;
	}
	public void mouseClicked(MouseEvent e) {}
	public void mouseMoved(MouseEvent e) {}
	public void mouseEntered(MouseEvent e){	}
	public void mouseExited(MouseEvent e){	}

	
	

	//
	// keyboard action
	//
	public void keyPressed(KeyEvent e) {
		System.out.println("key pressed: " + e.getKeyCode());
		
		if (e.getKeyCode() == 48)
			World.arm.set_target_angle(0, World.arm.get_current_angle(0)+10);
		if (e.getKeyCode() == 49)			
			World.arm.set_target_angle(1, World.arm.get_current_angle(1)+10);
		if (e.getKeyCode() == 50)			
			World.arm.set_target_angle(2, World.arm.get_current_angle(2)+10);
		if (e.getKeyCode() == 51)			
			World.arm.set_target_angle(3, World.arm.get_current_angle(3)+10);
		
			
	}
	public void keyReleased(KeyEvent e) {}
	public void keyTyped(KeyEvent e) {}

	
	
	
	
	//
	// do not change the code below this line ------------------
	//
	Main() {
		addMouseListener(this);
		addMouseMotionListener(this);
		addKeyListener(this);

		(new javax.swing.Timer(1000 / 30, this)).start();
	}

	public void actionPerformed(ActionEvent event) {
		repaint();
	}

	Image buffered_image = null;

	public void update(Graphics g) {
		paint(g);
	}

	public void paint(Graphics g) {
		// double buffering
		if (buffered_image == null)
			buffered_image = createImage(getSize().width, getSize().height);
		Graphics bg = buffered_image.getGraphics();

		// clear
		bg.setColor(Color.white);
		bg.fillRect(0, 0, getSize().width, getSize().height);

		// paint code!
		World.paint(bg);
		paint_feedback(bg);

		// timer
		if (count_time){
			elapsed_time = System.currentTimeMillis()-start_time;
			
			if (World.target_reached()){
				Applet.newAudioClip(Main.class.getResource("./sound.wav")).play();

				count_time = false;
			}
		}
		bg.setColor(Color.black);
		bg.drawString("time "+ elapsed_time/1000, 10, 20);

		// double buffering
		bg.dispose();

		g.drawImage(buffered_image, 0, 0, this);
	}
	
	static void start_timer(){
		count_time = true;
		start_time = System.currentTimeMillis();
	}
	static boolean count_time;
	static long elapsed_time;
	static long start_time = 0;

	public static void main(String args[]) {
		Frame f = new Frame("Main");

		f.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				System.exit(0);
			}
		});
		f.setMenuBar(new MyMenuBar());
		f.setLayout(new BorderLayout());
		Main main = new Main();
		f.add("Center", main);
		f.add("South", new Buttons(main));

		f.setSize(800, 700);
		f.setVisible(true);
	}
}
