package world;

import board.*;
import java.awt.*;
//
//do not modify
//

public class Arm {
	Vector2D base_position = new Vector2D(400, 300);
	double[] angles = { 0, 0, 0, 0 };
	double[] target_angles = { 0, 0, 0, 0 };
	Vector2D[] joint_positions;

	public static int N = 5;
	public static int L = 80;
	public static int R = 10;


	public double get_target_angle(int i){
		if (i<0 || i>=N-1)
			return -1;
		return target_angles[i];
	}
	public void set_target_angle(int i, double angle){
		if (i<0 || i>=N-1)
			return;
		target_angles[i] = angle;
	}
	public double get_current_angle(int i){
		if (i<0 || i>=N-1)
			return -1;
		return angles[i];
	}
	public Vector2D get_joint_position(int i) {
		if (i<0 || i>=N)
			return null;
		return new Vector2D(joint_positions[i]);
	}
	
	
	void compute_joint_positions() {
		double angle = 0;
		joint_positions = new Vector2D[N];
		joint_positions[0] = base_position;
		for (int i = 1; i < N; i++) {
			angle +=angles[i - 1];
			joint_positions[i] = Vector2D.add(joint_positions[i - 1],
					new Vector2D(L * Math.cos(Math.PI / 180 * angle), L
							* Math.sin(Math.PI / 180 * angle)));

		}
	}

	int dR = 2;
	void tick() {
		double[] prev_angles = angles.clone();

		for (int i = 0; i < N-1; i++) {
			if (angles[i] < target_angles[i]-dR)
				angles[i] += dR;
			else if (angles[i] > target_angles[i]+dR)
				angles[i] -= dR;
		}

		if (World.detect_collision()) {
			collision = true;
			angles = prev_angles;
		} else
			collision = false;

	}

	boolean collision = false;

	void paint(Graphics g) {
		compute_joint_positions();

		Graphics2D g2 = (Graphics2D) g;
		g2.setStroke(new BasicStroke(5));

		g2.setColor(Color.black);
		for (int i = 0; i < N - 1; i++) {
			Vector2D p0 = joint_positions[i];
			Vector2D p1 = joint_positions[i + 1];
			g2.drawLine((int) p0.x, (int) p0.y, (int) p1.x, (int) p1.y);
		}

		for (int i = 0; i < N; i++) {
			Vector2D p = joint_positions[i];
			g2.setColor(Color.white);
			g2.fillOval((int) p.x - R, (int) p.y - R, 2 * R, 2 * R);
			if (collision)
				g2.setColor(Color.red);
			else
				g2.setColor(Color.black);
			g2.drawOval((int) p.x - R, (int) p.y - R, 2 * R, 2 * R);
		}

	}

}
